// Copyright 2017 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

package com.qpc.qpc_admob;

import android.app.Activity;
import android.util.Log;
import android.util.SparseArray;
import android.view.Gravity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.LinearLayout;
import android.widget.Toast;

import com.bytedance.sdk.openadsdk.AdSlot;
import com.bytedance.sdk.openadsdk.TTAdManager;
import com.bytedance.sdk.openadsdk.TTAdNative;
import com.bytedance.sdk.openadsdk.TTAdSdk;
import com.bytedance.sdk.openadsdk.TTAppDownloadListener;
import com.bytedance.sdk.openadsdk.TTInteractionAd;
import com.bytedance.sdk.openadsdk.TTNativeExpressAd;
import com.bytedance.sdk.openadsdk.TTNativeExpressAd.ExpressAdInteractionListener;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import io.flutter.plugin.common.MethodChannel;

abstract class MobileAd implements ExpressAdInteractionListener, TTAdNative.InteractionAdListener, TTNativeExpressAd.AdInteractionListener, TTAppDownloadListener {
  private static final String TAG = "flutter";
  private static SparseArray<MobileAd> allAds = new SparseArray<MobileAd>();

  final Activity activity;
  final MethodChannel channel;
  final int id;
  Status status;
  double anchorOffset;
  double horizontalCenterOffset;
  int anchorType;
  TTAdNative mTTAdNative;
  TTInteractionAd ttInteractionAd;

  enum Status {
    CREATED,
    LOADING,
    FAILED,
    PENDING, // The ad will be shown when status is changed to LOADED.
    LOADED,
  }

  private MobileAd(int id, Activity activity, MethodChannel channel) {
    this.id = id;
    this.activity = activity;
    this.channel = channel;
    this.status = Status.CREATED;
    this.anchorOffset = 0.0;
    this.horizontalCenterOffset = 0.0;
    this.anchorType = Gravity.BOTTOM;
    allAds.put(id, this);
  }

  static Banner createBanner(Integer id,String adUnitId, int width, int height, Activity activity, MethodChannel channel) {
    MobileAd ad = getAdForId(id);
    return (ad != null) ? (Banner) ad : new Banner(id,width,height,activity,channel);

  }

  static Interstitial createInterstitial(Integer id, Activity activity, MethodChannel channel) {
    MobileAd ad = getAdForId(id);
    return (ad != null) ? (Interstitial) ad : new Interstitial(id, activity, channel);
  }

  static MobileAd getAdForId(Integer id) {
    return allAds.get(id);
  }

  Status getStatus() {
    return status;
  }

  abstract void load(String adUnitId, Map<String, Object> targetingInfo);

  abstract void show();

  void dispose() {
    allAds.remove(id);
  }

  private Map<String, Object> argumentsMap(Object... args) {
    Map<String, Object> arguments = new HashMap<String, Object>();
    arguments.put("id", id);
    for (int i = 0; i < args.length; i += 2) arguments.put(args[i].toString(), args[i + 1]);
    return arguments;
  }

  @Override
  public void onAdClicked(View view, int type) {
//    TToast.show(mContext, "广告被点击");
    channel.invokeMethod("onAdClicked", argumentsMap());
  }

  @Override
  public void onAdShow(View view, int type) {
//    TToast.show(mContext, "广告展示");
    boolean statusWasPending = status == Status.PENDING;
    status = Status.LOADED;
    channel.invokeMethod("onAdLoaded", argumentsMap());
    if (statusWasPending) show();

  }

  @Override
  public void onAdDismiss() {
    Log.d(TAG, "插屏广告消失");
    TToast.show(activity, "广告消失");
  }

  @Override
  public void onRenderFail(View view, String msg, int code) {
    Log.w(TAG, msg + code);
    status = Status.FAILED;
    channel.invokeMethod("onAdFailedToLoad", argumentsMap("errorMsg",msg,"errorCode", code));
  }

  @Override
  public void onRenderSuccess(View view, float width, float height) {
    //返回view的宽高 单位 dp
//    TToast.show(mContext, "渲染成功");
    //在渲染成功回调时展示广告，提升体验
//    mExpressContainer.removeAllViews();
//    mExpressContainer.addView(view);
  }

  @Override
  public void onIdle() {
    TToast.show(activity, "点击开始下载", Toast.LENGTH_LONG);
  }

  @Override
  public void onDownloadActive(long totalBytes, long currBytes, String fileName, String appName) {

      TToast.show(activity, "下载中，点击暂停", Toast.LENGTH_LONG);

  }

  @Override
  public void onDownloadPaused(long totalBytes, long currBytes, String fileName, String appName) {
    TToast.show(activity, "下载暂停，点击继续", Toast.LENGTH_LONG);
  }

  @Override
  public void onDownloadFailed(long totalBytes, long currBytes, String fileName, String appName) {
    TToast.show(activity, "下载失败，点击重新下载", Toast.LENGTH_LONG);
  }

  @Override
  public void onInstalled(String fileName, String appName) {
    TToast.show(activity, "安装完成，点击图片打开", Toast.LENGTH_LONG);
  }

  @Override
  public void onDownloadFinished(long totalBytes, String fileName, String appName) {
    TToast.show(activity, "点击安装", Toast.LENGTH_LONG);
  }


  @Override
  public void onError(int code, String message) {
    TToast.show(activity, "load error : " + code + ", " + message);
  }

  @Override
  public void onInteractionAdLoad(TTInteractionAd ttInteractionAd) {
    TToast.show(activity, "type:  " + ttInteractionAd.getInteractionType());
    ttInteractionAd.setAdInteractionListener((TTInteractionAd.AdInteractionListener) this);
    ttInteractionAd.setDownloadListener(this);
    this.ttInteractionAd = ttInteractionAd;
    ttInteractionAd.showInteractionAd(activity);

  }

  static class Banner extends MobileAd {

    private  int width;
    private int height;

    private Banner(Integer id, int width, int height, Activity activity, MethodChannel channel) {
      super(id, activity, channel);
      this.width=width;
      this.height=height;
      //step5:请求广告
      TTAdManager ttAdManager =  TTAdSdk.getAdManager();
      //step2:(可选，强烈建议在合适的时机调用):申请部分权限，如read_phone_state,防止获取不了imei时候，下载类广告没有填充的问题。
      ttAdManager.requestPermissionIfNecessary(activity);
      //step3:创建TTAdNative对象,用于调用广告请求接口
      mTTAdNative = ttAdManager.createAdNative(activity);

    }

    @Override
    void load(String adUnitId, Map<String, Object> targetingInfo) {
      if (status != Status.CREATED) return;
      status = Status.LOADING;
      AdSlot adSlot = new AdSlot.Builder()
              .setCodeId(adUnitId)
              .setSupportDeepLink(true)
              .setImageAcceptedSize(width, height)
              .setAdCount(1)
              .build();

      mTTAdNative.loadBannerAd(adSlot, (TTAdNative.BannerAdListener) this);

    }

    @Override
    void show() {
      if (status == Status.LOADING) {
        status = Status.PENDING;
        return;
      }
      if (status != Status.LOADED) return;

      if (activity.findViewById(id) == null) {
//        LinearLayout content = new LinearLayout(activity);
//        content.setId(id);
//        content.setOrientation(LinearLayout.VERTICAL);
//        content.setGravity(anchorType);
//        content.addView(adView);
//        final float scale = activity.getResources().getDisplayMetrics().density;
//
//        int left = horizontalCenterOffset > 0 ? (int) (horizontalCenterOffset * scale) : 0;
//        int right =
//            horizontalCenterOffset < 0 ? (int) (Math.abs(horizontalCenterOffset) * scale) : 0;
//        if (anchorType == Gravity.BOTTOM) {
//          content.setPadding(left, 0, right, (int) (anchorOffset * scale));
//        } else {
//          content.setPadding(left, (int) (anchorOffset * scale), right, 0);
//        }
//
//        activity.addContentView(
//            content,
//            new ViewGroup.LayoutParams(
//                ViewGroup.LayoutParams.MATCH_PARENT, ViewGroup.LayoutParams.MATCH_PARENT));
      }
    }

    @Override
    void dispose() {
      super.dispose();


      View contentView = activity.findViewById(id);
      if (contentView == null || !(contentView.getParent() instanceof ViewGroup)) return;

      ViewGroup contentParent = (ViewGroup) (contentView.getParent());
      contentParent.removeView(contentView);
      mTTAdNative=null;
    }
  }

  static class Interstitial extends MobileAd {

    private Interstitial(int id, Activity activity, MethodChannel channel) {
      super(id, activity, channel);
      //step5:请求广告
      TTAdManager ttAdManager =  TTAdSdk.getAdManager();
      //step2:(可选，强烈建议在合适的时机调用):申请部分权限，如read_phone_state,防止获取不了imei时候，下载类广告没有填充的问题。
      ttAdManager.requestPermissionIfNecessary(activity);
      //step3:创建TTAdNative对象,用于调用广告请求接口
       mTTAdNative = ttAdManager.createAdNative(activity);
    }

    @Override
    void load(String adUnitId, Map<String, Object> targetingInfo) {
      status = Status.LOADING;

      AdSlot adSlot = new AdSlot.Builder()
              .setCodeId(adUnitId)
              .setSupportDeepLink(true)
              .setAdCount(1)
              .build();
      mTTAdNative.loadInteractionExpressAd(adSlot, (TTAdNative.NativeExpressAdListener) this);
    }

    @Override
    void show() {
      if (status == Status.LOADING) {
        status = Status.PENDING;
        return;
      }
      if(ttInteractionAd !=null ){
        ttInteractionAd.showInteractionAd(activity);
      }
    }

    // It is not possible to hide/remove/destroy an AdMob interstitial Ad.
  }
}
